﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Xml;
using System.IO;

namespace Lab3Client
{
    public partial class Form1 : Form
    {
        // data
        Socket serverSocket;
        IPEndPoint ipepServer;
        bool isConnected = false;
        byte[] data = new byte[1024 * 1024];

        public Form1()
        {
            InitializeComponent();
        }

        private void buttonConnect_Click(object sender, EventArgs e)
        {
            string port = inputPort.Text;
            string hostname = inputHostname.Text;
            if (isConnected)
            {
                try
                {
                    serverSocket.Close();
                    isConnected = false;
                    buttonConnect.Text = "Connect";
                    label3.Visible = false;
                    path.Visible = false;
                    ShowDirectory("clear");
                } 
                catch
                {
                    MessageBox.Show(
                        "Can not close connection",
                        "Error");
                }
            }
            else
            {
                try
                {

                    ipepServer = new IPEndPoint(
                        IPAddress.Parse(hostname), Int32.Parse(port));

                    serverSocket = new Socket(AddressFamily.InterNetwork,
                        SocketType.Stream, ProtocolType.Tcp);

                    serverSocket.Connect(ipepServer);

                    isConnected = true;

                    buttonConnect.Text = "Disconnect";

                    label3.Visible = true;
                    path.Visible = true;
                    ShowDirectory("/");
                }
                catch
                {
                    MessageBox.Show(
                        "Can not connect to server " + hostname + ":" + port,
                        "Error");
                }
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (isConnected)
            {
                serverSocket.Close();
            }
        }

        public void ShowDirectory(string dirPath)
        {
            if (dirPath == "clear")
            {
                fileList.Items.Clear();
                return;
            }
            if (!isConnected)
            {
                MessageBox.Show("Client isn't connected to server", "Error");
                return;
            }

            // <?xml ... ?>
            XmlDocument doc = new XmlDocument();
            XmlNode docNode = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
            doc.AppendChild(docNode);

            XmlNode actionNode = doc.CreateElement("action");
            doc.AppendChild(actionNode);

            XmlNode commandNode = doc.CreateElement("command");
            commandNode.InnerText = "showDir";
            actionNode.AppendChild(commandNode);

            XmlNode pathNode = doc.CreateElement("path");
            pathNode.InnerText = dirPath;
            actionNode.AppendChild(pathNode);
            path.Text = dirPath;

            serverSocket.Send(Encoding.UTF8.GetBytes(doc.OuterXml));

            int datalen = serverSocket.Receive(data);
            string str = Encoding.UTF8.GetString(data, 0, datalen);
           
            if (str == "error")
            {
                MessageBox.Show("Server unknown error", "Error");
                return;
            }
            try
            {
                doc.LoadXml(str);

                XmlNodeList list = doc.SelectNodes("files/file");
                fileList.Items.Clear();


                if (path.Text != "/")
                {
                    fileList.Items.Add("../");
                }

                foreach (XmlNode node in list)
                {
                    fileList.Items.Add(node.InnerText);
                }
            }
            catch
            {
                MessageBox.Show("Server unknown error", "Error");
                return;
            }
        }

        private void buttonAddDirectory_Click(object sender, EventArgs e)
        {
            if (!isConnected)
            {
                MessageBox.Show("Client isn't connected to server", "Error");
                return;
            }

            // <?xml ... ?>
            XmlDocument doc = new XmlDocument();
            XmlNode docNode = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
            doc.AppendChild(docNode);

            XmlNode actionNode = doc.CreateElement("action");
            doc.AppendChild(actionNode);

            XmlNode commandNode = doc.CreateElement("command");
            commandNode.InnerText = "addDir";
            actionNode.AppendChild(commandNode);

            XmlNode pathNode = doc.CreateElement("path");
            pathNode.InnerText = path.Text;
            actionNode.AppendChild(pathNode);

            XmlNode nameNode = doc.CreateElement("name");
            string name = "New folder";
            DialogResult res = InputBox("New folder", "Please, input new folder name", ref name);
            if (res != DialogResult.OK)
            {
                return;
            }
            nameNode.InnerText = name + "/";
            actionNode.AppendChild(nameNode);

            serverSocket.Send(Encoding.UTF8.GetBytes(doc.OuterXml));
           
            int datalen = serverSocket.Receive(data);

            if (Encoding.UTF8.GetString(data, 0, datalen) == "error")
            {
                MessageBox.Show("Server unknown error", "Error");
                return;
            }
            if (Encoding.UTF8.GetString(data, 0, datalen) == "false")
            {
                MessageBox.Show("Folder already exists", "Error");
                return;
            }

            ShowDirectory(path.Text);

        }

        private void buttonRename_Click(object sender, EventArgs e)
        {
            if (!isConnected)
            {
                MessageBox.Show("Client isn't connected to server", "Error");
                return;
            }

            if (fileList.SelectedIndex == -1)
            {
                MessageBox.Show("Please, Choose file or folder", "Error");
                return;
            }

            // <?xml ... ?>
            XmlDocument doc = new XmlDocument();
            XmlNode docNode = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
            doc.AppendChild(docNode);

            XmlNode actionNode = doc.CreateElement("action");
            doc.AppendChild(actionNode);

            XmlNode commandNode = doc.CreateElement("command");
            commandNode.InnerText = "rename";
            actionNode.AppendChild(commandNode);

            XmlNode pathNode = doc.CreateElement("path");
            pathNode.InnerText = path.Text;
            actionNode.AppendChild(pathNode);

            XmlNode nameNode = doc.CreateElement("name");
            string name = (string) fileList.SelectedItem;
            nameNode.InnerText = name;
            actionNode.AppendChild(nameNode);

            XmlNode newnameNode = doc.CreateElement("newname");
            string newname = "";
            DialogResult res = InputBox("Change name", "Please, input new name", ref newname);
            if (res != DialogResult.OK)
            {
                return;
            }
            if (name.Contains('/'))
            {
                newnameNode.InnerText = newname + "/";
            } else {
                newnameNode.InnerText = newname;
            }
            if (newname == "") {
                MessageBox.Show("Please, Input new name", "Error");
                return;                
            }
           
            actionNode.AppendChild(newnameNode);

            serverSocket.Send(Encoding.UTF8.GetBytes(doc.OuterXml));

            int datalen = serverSocket.Receive(data);

            if (Encoding.UTF8.GetString(data, 0, datalen) == "error")
            {
                MessageBox.Show("Server unknown error", "Error");
                return;
            }

            ShowDirectory(path.Text);
        }

        private void buttonRemove_Click(object sender, EventArgs e)
        {
            if (!isConnected)
            {
                MessageBox.Show("Client isn't connected to server", "Error");
                return;
            }
            
            if (fileList.SelectedIndex == -1)
            {
                MessageBox.Show("Please, Choose file or folder", "Error");
                return;
            }

            // <?xml ... ?>
            XmlDocument doc = new XmlDocument();
            XmlNode docNode = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
            doc.AppendChild(docNode);

            XmlNode actionNode = doc.CreateElement("action");
            doc.AppendChild(actionNode);

            XmlNode commandNode = doc.CreateElement("command");
            commandNode.InnerText = "remove";
            actionNode.AppendChild(commandNode);

            XmlNode pathNode = doc.CreateElement("path");
            pathNode.InnerText = path.Text;
            actionNode.AppendChild(pathNode);

            XmlNode nameNode = doc.CreateElement("name");
            string name = (string) fileList.SelectedItem;
            nameNode.InnerText = name;
            actionNode.AppendChild(nameNode);

            serverSocket.Send(Encoding.UTF8.GetBytes(doc.OuterXml));

            int datalen = serverSocket.Receive(data);

            if (Encoding.UTF8.GetString(data, 0, datalen) == "error")
            {
                MessageBox.Show("Server unknown error", "Error");
                return;
            }

            ShowDirectory(path.Text);

        }

        private void buttonUpload_Click(object sender, EventArgs e)
        {
            if (!isConnected)
            {
                MessageBox.Show("Client isn't connected to server", "Error");
                return;
            }

            // <?xml ... ?>
            XmlDocument doc = new XmlDocument();
            XmlNode docNode = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
            doc.AppendChild(docNode);

            XmlNode actionNode = doc.CreateElement("action");
            doc.AppendChild(actionNode);

            XmlNode commandNode = doc.CreateElement("command");
            commandNode.InnerText = "upload";
            actionNode.AppendChild(commandNode);

            XmlNode pathNode = doc.CreateElement("path");
            pathNode.InnerText = path.Text;
            actionNode.AppendChild(pathNode);

            OpenFileDialog dialog = new OpenFileDialog();
            if (dialog.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            Stream stream = dialog.OpenFile();
            
            XmlNode nameNode = doc.CreateElement("name");
            nameNode.InnerText = dialog.FileName;
            int index = nameNode.InnerText.LastIndexOf('\\');
            nameNode.InnerText = nameNode.InnerText.Substring(index + 1);
            actionNode.AppendChild(nameNode);

            XmlNode dataNode = doc.CreateElement("data");
            data = new byte[1024 * 1024];
            int len = stream.Read(data, 0, 1024 * 1024);
            dataNode.InnerText = Convert.ToBase64String(data, 0, len);
            actionNode.AppendChild(dataNode);
            serverSocket.Send(Encoding.UTF8.GetBytes(doc.OuterXml));

            int datalen = serverSocket.Receive(data);

            if (Encoding.UTF8.GetString(data, 0, datalen) == "error")
            {
                MessageBox.Show("Server unknown error", "Error");
                return;
            }

            stream.Close();


            ShowDirectory(path.Text);
        }
     
        private void buttonDownload_Click(object sender, EventArgs e)
        {
            if (!isConnected)
            {
                MessageBox.Show("Client isn't connected to server", "Error");
                return;
            }
            if (fileList.SelectedIndex == -1)
            {
                MessageBox.Show("Please, Choose file to download", "Error");
                return;
            }

            XmlDocument doc = new XmlDocument();
            XmlNode docNode = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
            doc.AppendChild(docNode);

            XmlNode actionNode = doc.CreateElement("action");
            doc.AppendChild(actionNode);

            XmlNode commandNode = doc.CreateElement("command");
            commandNode.InnerText = "download";
            actionNode.AppendChild(commandNode);

            XmlNode pathNode = doc.CreateElement("path");
            pathNode.InnerText = path.Text;
            actionNode.AppendChild(pathNode);

            XmlNode nameNode = doc.CreateElement("name");
            nameNode.InnerText = (string) fileList.SelectedItem;
            actionNode.AppendChild(nameNode);

            serverSocket.Send(Encoding.UTF8.GetBytes(doc.OuterXml));

            int datalen = serverSocket.Receive(data);

            if (Encoding.UTF8.GetString(data, 0, datalen) == "error")
            {
                MessageBox.Show("Server unknown error", "Error");
                return;
            }

            SaveFileDialog dialog = new SaveFileDialog();
            dialog.FileName = (string) fileList.SelectedItem;
            if (dialog.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            doc.LoadXml(Encoding.UTF8.GetString(data, 0, datalen));
            byte[] convertdata = Convert.FromBase64String(doc.SelectSingleNode("file/data").InnerText);
            
           
            Stream s = dialog.OpenFile();
            s.Write(convertdata, 0, convertdata.Length);
            s.Close();

            ShowDirectory(path.Text);
        }

        private void fileList_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        public static DialogResult InputBox(string title, string promptText, ref string value)
        {
            Form form = new Form();
            Label label = new Label();
            TextBox textBox = new TextBox();
            Button buttonOk = new Button();
            Button buttonCancel = new Button();

            form.Text = title;
            label.Text = promptText;
            textBox.Text = value;

            buttonOk.Text = "OK";
            buttonCancel.Text = "Cancel";
            buttonOk.DialogResult = DialogResult.OK;
            buttonCancel.DialogResult = DialogResult.Cancel;

            label.SetBounds(9, 20, 372, 13);
            textBox.SetBounds(12, 36, 372, 20);
            buttonOk.SetBounds(228, 72, 75, 23);
            buttonCancel.SetBounds(309, 72, 75, 23);

            label.AutoSize = true;
            textBox.Anchor = textBox.Anchor | AnchorStyles.Right;
            buttonOk.Anchor = AnchorStyles.Bottom | AnchorStyles.Right;
            buttonCancel.Anchor = AnchorStyles.Bottom | AnchorStyles.Right;

            form.ClientSize = new Size(396, 107);
            form.Controls.AddRange(new Control[] { label, textBox, buttonOk, buttonCancel });
            form.ClientSize = new Size(Math.Max(300, label.Right + 10), form.ClientSize.Height);
            form.FormBorderStyle = FormBorderStyle.FixedDialog;
            form.StartPosition = FormStartPosition.CenterScreen;
            form.MinimizeBox = false;
            form.MaximizeBox = false;
            form.AcceptButton = buttonOk;
            form.CancelButton = buttonCancel;

            DialogResult dialogResult = form.ShowDialog();
            value = textBox.Text;
            return dialogResult;
        }

        private void fileList_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (fileList.SelectedIndex == -1)
            {
                return;
            }
            string item = fileList.SelectedItem.ToString();
            
            // check - is folder?
            if (item.Contains('/'))
            {
                // folder
                if (item == "../")
                {
                    string subpath = path.Text.Substring(0, path.Text.Length - 1);
                    int pos = subpath.LastIndexOf('/') + 1;
                    ShowDirectory(subpath.Substring(0, pos));
                }
                else
                {
                    ShowDirectory(path.Text + item);
                }
            }
            else
            {
                buttonDownload_Click(sender, e);
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (!isConnected)
            {
                MessageBox.Show("Client isn't connected to server", "Error");
                return;
            }

            // <?xml ... ?>
            XmlDocument doc = new XmlDocument();
            XmlNode docNode = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
            doc.AppendChild(docNode);

            XmlNode actionNode = doc.CreateElement("action");
            doc.AppendChild(actionNode);

            XmlNode commandNode = doc.CreateElement("command");
            commandNode.InnerText = "task";
            actionNode.AppendChild(commandNode);

            string tmpstr = "n";
            XmlNode startNode = doc.CreateElement("path");
            if (DialogResult.OK != InputBox("InputBox","Input beginnig of file", ref tmpstr)) {
                return;
            }
            startNode.InnerText = tmpstr;
            actionNode.AppendChild(startNode);

            serverSocket.Send(Encoding.UTF8.GetBytes(doc.OuterXml));

            int datalen = serverSocket.Receive(data);
            string str = Encoding.UTF8.GetString(data, 0, datalen);

            if (str == "error")
            {
                MessageBox.Show("Server unknown error", "Error");
                return;
            }
            try
            {
                doc.LoadXml(str);

                XmlNodeList list = doc.SelectNodes("files/file");

                StringBuilder builder = new StringBuilder();
                foreach (XmlNode node in list)
                {
                    builder.Append(node.InnerText);
                    builder.Append("\n");
                }
                MessageBox.Show(builder.ToString(), "Results");
            }
            catch
            {
                MessageBox.Show("Server unknown error", "Error");
                return;
            }
        }
        
    }
}
